function love.load()
  love.filesystem.setIdentity('screenshot')
  units = 16
  myFont = love.graphics.newFont(20,"mono")
  --libs
  physics = require("libs/physics-master/physics") --simple box2d physics wrapper
  myWorld = physics()
  bitser = require("libs/bitser-master/bitser") --saving and loading tables
  anim8 = require("libs/anim8-master/anim8") --animations
  sti = require("libs/sti-master/sti") --tiled map loader and renderer
  moonshine = require("libs/moonshine-master") --shader pipeline effects
  effect = moonshine(moonshine.effects.dmg)
              .chain(moonshine.effects.crt)
                  .chain(moonshine.effects.scanlines)
  local chocoboy = {
    {0.0,0.1,0.2},
    {0.3,0.1,0.4},
    {0.6,0.4,0.3},
    {1.0,0.9,0.8}
  }
  effect.dmg.palette = chocoboy
  effect.scanlines.opacity = 0.4
      map = sti("maps/canari16x16.lua")
  push = require("libs/push-master/push") --fixed screen resolution
      screen_w = love.graphics.getWidth()
      screen_h = love.graphics.getHeight()
      desktop_w, desktop_h = love.window.getDesktopDimensions()
      desktop_w, desktop_h = desktop_w * 0.7, desktop_h * 0.7
      push:setupScreen(screen_w, screen_h, desktop_w, desktop_h, {fullscreen = false, resizable = true})
  gamera = require("libs/gamera-master/gamera") --easy camrea setup
      cam = gamera.new((-screen_w/4),(-screen_h/4),screen_w*2,screen_h*2)
      cam:setWindow(0,0,screen_w*1,screen_h*1)
      cam:setScale(4.0) --2 or greater
  --show testing control table
  show = {}
  show.line = false
  show.world = false
  --user scripts
  require("sprites") --sprites and animations
  require("collide") --multiple physics collision objects
  require("objects") --physics single object
  require("player") --player objcet and state
  require("worldstate") --myWorld collision checks
  --remove unneeded object layers
  map:removeLayer("POS")
end

function love.update(dt)
  map:update(dt)
  myWorld:update(dt)
  for i = 1, 4 do
    hero[i]:update(dt) --update all posible directions 1 to 4
  end
  --toogle fullscreen
  if love.keyboard.isDown("ralt") and love.keyboard.isDown("return") then
    push:switchFullscreen() --does not reset screen size when windowed
  end
  if love.keyboard.isDown("lalt") and love.keyboard.isDown("return") then
    push:switchFullscreen(screen_w, screen_h) --reset screen size when windowed
  end
end

function love.resize(w, h)
  push:resize(w, h)
end

function love.keypressed(key, scancode, isrepeat)
  --quit game
  if key == "escape" then
    love.event.quit()
  end
  --reatart game
  if key == "backspace" then
    love.event.quit("restart")
  end
  if key == "c" then
    love.graphics.captureScreenshot(os.time() .. ".png")
  end
end

function love.draw()
  --game world
  push:start()
  effect(function()
    cam:draw(function()
        local p = map.layers["Player"].player
        map:drawLayer(map.layers["D2"])
        object_shadows()
        player_shadows()
        map:drawLayer(map.layers["D1"])
        map:drawLayer(map.layers["ObjectsDown"])
        map:drawLayer(map.layers["ObjectsUp"])
        map:drawLayer(map.layers["Player"])
        map:drawLayer(map.layers["U1"])
        map:drawLayer(map.layers["U2"])
      if show.world then --testing physics world
        myWorld:draw()
      end
    end)
  end)
  --overhead UI
  if show.line then
      love.graphics.setColor(1,0,0)
      love.graphics.line(screen_w/2,0,screen_w/2,screen_h)
      love.graphics.line(0,screen_h/2,screen_w,screen_h/2)
      love.graphics.setColor(1,1,1)
  end
  if show.world then
      love.graphics.setFont(myFont)
      love.graphics.print("player state: "..p.player.state,0,0)
  end
  push:finish()
end
